"use strict";

require("core-js/modules/es.symbol.js");

require("core-js/modules/es.symbol.description.js");

require("core-js/modules/es.array.filter.js");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.array.map.js");

require("core-js/modules/es.object.get-own-property-descriptors.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.string.split.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = moduleExtensionResolver;

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var PLUGIN_NAME = "babel-plugin-module-extension-resolver";
var DEFAULT_EXTENSIONS = [".js", ".cjs", ".mjs", ".es", ".es6", ".ts", ".node", ".json"];
var DEFAULT_MAP = {
  ".ts": ".js",
  ".es": ".js",
  ".es6": ".js",
  ".node": ".js"
};

function moduleExtensionResolver(babel, options) {
  var types = babel.types;

  var normalizedOptions = _objectSpread({
    extensions: DEFAULT_EXTENSIONS,
    map: DEFAULT_MAP
  }, options);

  return {
    name: PLUGIN_NAME,
    visitor: {
      Program: {
        enter(programPath, state) {
          var filename = state.file.opts.filename;
          programPath.traverse({
            CallExpression(declaration) {
              handleCallExpression(types, declaration, filename, normalizedOptions);
            },

            ImportDeclaration(declaration) {
              handleImportDeclaration(types, declaration, filename, normalizedOptions);
            },

            ExportDeclaration(declaration) {
              handleExportDeclaration(types, declaration, filename, normalizedOptions);
            }

          }, state);
        }

      }
    }
  };
}

function handleCallExpression(types, declaration, filename, options) {
  var callee = declaration.get("callee");

  if (!types.isIdentifier(callee) || callee.node.name !== "require") {
    return;
  }

  var args = declaration.get("arguments");

  if (args.length !== 1) {
    return;
  }

  replaceSource(types, args[0], filename, options);
}

function handleImportDeclaration(types, declaration, filename, options) {
  replaceSource(types, declaration.get("source"), filename, options);
}

function handleExportDeclaration(types, declaration, filename, options) {
  replaceSource(types, declaration.get("source"), filename, options);
}

function replaceSource(types, source, fileName, options) {
  if (!types.isStringLiteral(source)) {
    return;
  }

  var sourcePath = source.node.value;

  if (sourcePath[0] !== ".") {
    return;
  }

  var baseDir = _path.default.dirname(fileName);

  var resolvedPath = resolvePath(baseDir, sourcePath, options);
  var normalizedPath = normalizePath(resolvedPath);
  source.replaceWith(types.stringLiteral(normalizedPath));
}

function resolvePath(baseDir, sourcePath, options) {
  var extensions = options.extensions,
      map = options.map;
  {
    var resolvedPath = resolvePathCore(baseDir, sourcePath, extensions, map);

    if (resolvedPath !== null) {
      return resolvedPath;
    }
  }

  var absolutePath = _path.default.join(baseDir, sourcePath);

  if (isDirectory(absolutePath)) {
    var _resolvedPath = resolvePathCore(baseDir, _path.default.join(sourcePath, "index"), extensions, map);

    if (_resolvedPath !== null) {
      return _resolvedPath;
    }
  }

  return sourcePath;
}

function resolvePathCore(baseDir, sourcePath, extensions, map) {
  var absolutePath = _path.default.join(baseDir, sourcePath);

  if (isFile(absolutePath)) {
    return sourcePath;
  }

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = extensions[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var extension = _step.value;
      var resolvedPath = `${absolutePath}${extension}`;

      if (!isFile(resolvedPath)) {
        continue;
      }

      if (map.hasOwnProperty(extension)) {
        return _path.default.relative(baseDir, `${absolutePath}${map[extension]}`);
      } else {
        return _path.default.relative(baseDir, resolvedPath);
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return null;
}

function normalizePath(originalPath) {
  if (_path.default.sep === "\\") {
    originalPath = originalPath.split(_path.default.sep).join("/");
  }

  if (originalPath[0] !== ".") {
    originalPath = `./${originalPath}`;
  }

  return originalPath;
}

function isFile(pathName) {
  try {
    return _fs.default.statSync(pathName).isFile();
  } catch (err) {
    return false;
  }
}

function isDirectory(pathName) {
  try {
    return _fs.default.statSync(pathName).isDirectory();
  } catch (err) {
    return false;
  }
}
//# sourceMappingURL=index.js.map